#include "program.h"
#include <SDL2/SDL_video.h>

struct Program g_program;

#define INITFLAGS SDL_INIT_EVENTS | SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_GAMECONTROLLER

int program_run(const struct ProgramSettings* settings) {
    SDL_Init(INITFLAGS);

    g_program.window = SDL_CreateWindow(
        settings->title,
        SDL_WINDOWPOS_CENTERED_DISPLAY(0),
        SDL_WINDOWPOS_CENTERED_DISPLAY(0),
        settings->view_resolution.x,
        settings->view_resolution.y,
        SDL_WINDOW_FULLSCREEN | SDL_WINDOW_RESIZABLE);

    g_program.render_context = render_init(&g_program, settings);

    for(;;) {
        SDL_Delay(1);
        program_handle_events(&g_program);
        render_present(g_program.render_context);
    }

    return 1;
}

void program_quit(struct Program* self) {
    render_destroy(self->render_context);
    SDL_DestroyWindow(self->window);
    SDL_Quit();
    exit(0);
}

void program_handle_events(struct Program* self) {
    SDL_Event event;
    while(SDL_PollEvent(&event)) {
        switch(event.type) {
            default: break;
            case SDL_WINDOWEVENT:
                if(event.window.windowID == SDL_GetWindowID(self->window)) {
                    program_handle_windowevent(self, &event.window);
                }
                break;
            case SDL_QUIT:
                program_quit(self);
                break;
        }
    }
}

void program_handle_windowevent(struct Program* self, SDL_WindowEvent* event) {
    switch(event->type) {
        default:
            render_handle_resize(self->render_context);
            break;
    }
}
